@@ -1,8 +1,8 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-#        Copyright (C) 2009 Daniel Fett
-#         This program is free software: you can redistribute it and/or modify
+#   Copyright (C) 2010 Daniel Fett
+#   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation, either version 3 of the License, or
 #   (at your option) any later version.
@@ -15,14 +15,20 @@
 #   You should have received a copy of the GNU General Public License
 #   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-#        Author: Daniel Fett advancedcaching@fragcom.de
+#   Author: Daniel Fett agtl@danielfett.de
+#   Jabber: fett.daniel@jaber.ccc.de
+#   Bugtracker and GIT Repository: http://github.com/webhamster/advancedcaching
 #
 
 from __future__ import with_statement
 
-VERSION = "0.7.0.2"
+VERSION = "0.8.0.0"
    
 
+import logging
+logging.basicConfig(level=logging.WARNING,
+                    format='%(relativeCreated)6d %(levelname)10s %(name)-20s %(message)s',
+                    )
 
 from geo import Coordinate
 try:
@@ -43,11 +49,6 @@ import cachedownloader
 import fieldnotesuploader
 from actors.tts import TTS
 #from actors.notify import Notify
-import logging
-
-logging.basicConfig(level=logging.WARNING,
-                    format='%(asctime)s\t%(levelname)s\t%(name)-15s\t %(message)s',
-                    )
 
 if len(argv) == 1:
     import cli
@@ -55,46 +56,54 @@ if len(argv) == 1:
     exit()
 
 if '-v' in argv:
+    import colorer
+    import logging.handlers
     logging.getLogger('').setLevel(logging.DEBUG)
     logging.debug("Set log level to DEBUG")
 
-arg = argv[1].strip()
-if arg == '--simple':
+if '--simple' in argv:
     import simplegui
     gui = simplegui.SimpleGui
-elif arg == '--desktop':
+elif '--desktop' in argv:
     import biggui
     gui = biggui.BigGui
-elif arg == '--hildon':
+elif '--qt' in argv:
+    import qtgui
+    gui = qtgui.QtGui
+elif '--hildon' in argv:
     import hildongui
     gui = hildongui.HildonGui
 else:
     import cli
     gui = cli.Cli
 
-    
+logger = logging.getLogger('core')
+
 class Core(gobject.GObject):
 
     __gsignals__ = {
         'map-marks-changed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ()),
-        'cache-changed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
+        'cache-changed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, )),
         'fieldnotes-changed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ()),
         'good-fix': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT)),
         'no-fix': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT)),
         'target-changed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT)),
-        'settings-changed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT,)),
+        'settings-changed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT, )),
         'save-settings': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ()),
+        'error': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, )),
+        'progress': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT, )),
+        'hide-progress': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ()),
         }
 
-    SETTINGS_DIR = path.expanduser('~/.agtl')
+    SETTINGS_DIR = path.expanduser(path.join('~', '.agtl'))
     CACHES_DB = path.join(SETTINGS_DIR, "caches.db")
     COOKIE_FILE = path.join(SETTINGS_DIR, "cookies.lwp")
     UPDATE_DIR = path.join(SETTINGS_DIR, 'updates')
 
-    MAEMO_HOME = path.expanduser("~/MyDocs/.")
-    MAPS_DIR = 'Maps/'
+    MAEMO_HOME = path.expanduser(path.join('~', 'MyDocs', '.'))
+    MAPS_DIR = path.join('Maps', '')
 
-    DATA_DIR = path.expanduser('~/') if not path.exists(MAEMO_HOME) else MAEMO_HOME
+    DATA_DIR = path.expanduser(path.join('~', '')) if not path.exists(MAEMO_HOME) else MAEMO_HOME
 
     UPDATE_MODULES = [cachedownloader, fieldnotesuploader]
     
@@ -106,7 +115,7 @@ class Core(gobject.GObject):
         'download_create_index': False,
         'download_run_after': False,
         'download_run_after_string': '',
-        'download_output_dir': path.expanduser(DATA_DIR + 'geocaches/'),
+        'download_output_dir': path.expanduser(path.join(DATA_DIR, 'geocaches', '')),
         'map_position_lat': 49.7540,
         'map_position_lon': 6.66135,
         'map_follow_position': True,
@@ -132,7 +141,7 @@ class Core(gobject.GObject):
         ],
         'options_map_double_size': False,
         'options_rotate_screen': 0,
-        'tts_interval' : 0
+        'tts_interval': 0
     }
             
     def __init__(self, guitype, root):
@@ -141,7 +150,7 @@ class Core(gobject.GObject):
         self.current_position = None
         self.create_recursive(self.SETTINGS_DIR)
 
-        dataroot = path.join(root, 'data')
+        self.dataroot = path.join(root, 'data')
 
         self._install_updates()
 
@@ -156,15 +165,14 @@ class Core(gobject.GObject):
                 
         self.pointprovider = provider.PointProvider(self.CACHES_DB, geocaching.GeocacheCoordinate, 'geocaches')
 
-        self.gui = guitype(self, dataroot)
+        self.gui = guitype(self, self.dataroot)
 
         
         actor_tts = TTS(self)
-        actor_tts.connect('error', lambda caller, msg: self.gui.show_error(msg))
+        actor_tts.connect('error', lambda caller, msg: self.emit('error', msg))
         #actor_notify = Notify(self)
-
         self.emit('settings-changed', self.settings, self)
-
+        self.emit('fieldnotes-changed')  
  
 
         if '--sim' in argv:
@@ -187,6 +195,8 @@ class Core(gobject.GObject):
 
 
         self.gui.show()
+        if not '--profile' in argv:
+            exit()
 
     ##############################################
     #
@@ -203,12 +213,12 @@ class Core(gobject.GObject):
                 try:
                     mkdir(dpath)
                 except Exception, e:
-                    logging.info("Could not create directory; " + e)
+                    logging.info("Could not create directory; %s" % e)
                     pass
 
     def optimize_data(self):
         self.pointprovider.push_filter()
-        self.pointprovider.set_filter(found = True)
+        self.pointprovider.set_filter(found=True)
         old_geocaches = self.pointprovider.get_points_filter()
         self.pointprovider.pop_filter()
         for x in old_geocaches:
@@ -228,11 +238,11 @@ class Core(gobject.GObject):
         folder = self.settings['download_output_dir']
         folder_size = 0
         for (p, dirs, files) in walk(folder):
-          for file in files:
-            filename = path.join(p, file)
-            folder_size += path.getsize(filename)
+            for file in files:
+                filename = path.join(p, file)
+                folder_size += path.getsize(filename)
         sqlite_size = path.getsize(self.CACHES_DB)
-        return {'images' : folder_size, 'sqlite' : sqlite_size}
+        return {'images': folder_size, 'sqlite': sqlite_size}
 
     @staticmethod
     def format_file_size(size):
@@ -337,42 +347,58 @@ class Core(gobject.GObject):
     #
     ##############################################
 
+    '''
+    This should be called to update the settings throughout all components.
+    '''
     def save_settings(self, settings, source):
+        logger.debug("Got settings update from %s" % source)
+        
         self.settings.update(settings)
         self.emit('settings-changed', settings, source)
 
+    '''
+    This is called when settings have changed.
+    '''
     def __on_settings_changed(self, caller, settings, source):
+        logger.debug("Settings where changed by %s." % source)
         if source == self:
             return
         if 'options_username' in settings and 'options_password' in settings:
             self.downloader.update_userdata(settings['options_username'], settings['options_password'])
 
+    '''
+    This is called when settings shall be saved, calling save_settings afterwards.
+    '''
     def __on_save_settings(self, caller):
+        logger.debug("Assembling update for settings, on behalf of %s" % caller)
         settings = {
             'last_target_lat': self.current_target.lat,
             'last_target_lon': self.current_target.lon
         }
         caller.save_settings(settings, self)
-                
+    
     def __del__(self):
+        logger.debug("Somebody is trying to kill me, saving the settings.")
         self.emit('save-settings')
         self.__write_config()
 
     # called by gui
     def on_destroy(self):
+        logger.debug("Somebody is being killed, saving the settings.")
         self.emit('save-settings')
         self.__write_config()
 
     # called by gui
-    def on_config_changed(self, new_settings):
-        self.settings = new_settings
-        self.downloader.update_userdata(self.settings['options_username'], self.settings['options_password'])
-        self.__write_config()
+    #def on_config_changed(self, new_settings):
+    #    self.settings = new_settings
+    #    self.downloader.update_userdata(self.settings['options_username'], self.settings['options_password'])
+    #    self.__write_config()
 
 
 
     def __read_config(self):
         filename = path.join(self.SETTINGS_DIR, 'config')
+        logger.debug("Loading config from %s" % filename)
         if not path.exists(filename):
             self.settings = self.DEFAULT_SETTINGS
             return
@@ -392,8 +418,10 @@ class Core(gobject.GObject):
 
     def __write_config(self):
         filename = path.join(self.SETTINGS_DIR, 'config')
+        logger.debug("Writing settings to %s" % filename)
         f = file(filename, 'w')
-        f.write(dumps(self.settings, sort_keys=True, indent=4))
+        config = dumps(self.settings, sort_keys=True, indent=4)
+        f.write(config)
 
 
     ##############################################
@@ -566,7 +594,7 @@ class Core(gobject.GObject):
 
     # called by gui
     def on_download(self, location, sync=False):
-        self.gui.set_download_progress(0.5, "Downloading...")
+        self.emit('progress', 0.5, "Downloading...")
         cd = cachedownloader.GeocachingComCacheDownloader(self.downloader, self.settings['download_output_dir'], not self.settings['download_noimages'])
         cd.connect("download-error", self.on_download_error)
         cd.connect("already-downloading-error", self.on_already_downloading_error)
@@ -577,7 +605,7 @@ class Core(gobject.GObject):
 
             cd.connect("finished-overview", same_thread)
             t = Thread(target=cd.get_geocaches, args=[location])
-            t.daemon = False
+            t.daemon = True
             t.start()
             return False
         else:
@@ -591,7 +619,7 @@ class Core(gobject.GObject):
             if point_new:
                 new_caches.append(c)
         self.pointprovider.save()
-        self.gui.hide_progress()
+        self.emit('hide-progress')
         self.emit('map-marks-changed')
         if sync:
             return (caches, new_caches)
@@ -600,21 +628,21 @@ class Core(gobject.GObject):
 
     # called on signal by downloading thread
     def on_already_downloading_error(self, something, error):
-        self.gui.show_error(error)
+        self.emit('error', error)
 
     # called on signal by downloading thread
     def on_download_error(self, something, error):
         logging.exception(error)
         def same_thread(error):
-            self.gui.hide_progress()
-            self.gui.show_error("Error while downloading: '%s'" % error)
+            self.emit('hide-progress')
+            self.emit('error', error)
             return False
         gobject.idle_add(same_thread, error)
 
     # called by gui
     def on_download_cache(self, cache, sync=False):
         #
-        self.gui.set_download_progress(0.5, "Downloading %s..." % cache.name)
+        self.emit('progress', 0.5, "Downloading %s..." % cache.name)
 
         cd = cachedownloader.GeocachingComCacheDownloader(self.downloader, self.settings['download_output_dir'], not self.settings['download_noimages'])
         cd.connect("download-error", self.on_download_error)
@@ -625,7 +653,7 @@ class Core(gobject.GObject):
                 return False
             cd.connect("finished-single", same_thread)
             t = Thread(target=cd.update_coordinate, args=[cache])
-            t.daemon = False
+            t.daemon = True
             t.start()
             #t.join()
             return False
@@ -637,7 +665,7 @@ class Core(gobject.GObject):
     def on_download_cache_complete(self, something, cache):
         self.pointprovider.add_point(cache, True)
         self.pointprovider.save()
-        self.gui.hide_progress()
+        self.emit('hide-progress')
         self.emit('cache-changed', cache)
         return False
 
@@ -688,11 +716,11 @@ class Core(gobject.GObject):
             gobject.idle_add(self.on_download_progress, arg1, arg2, arg3, arg4)
             return False
 
-        cd.connect('progress', same_thread_progress)
+        cd.connect('progress', self.on_download_progress)
         cd.connect('finished-multiple', same_thread)
 
         t = Thread(target=cd.update_coordinates, args=[caches])
-        t.daemon = False
+        t.daemon = True
         t.start()
 
 
@@ -701,7 +729,7 @@ class Core(gobject.GObject):
         for c in caches:
             self.pointprovider.add_point(c, True)
         self.pointprovider.save()
-        self.gui.hide_progress()
+        self.emit('hide-progress')
         for c in caches:
             self.emit('cache-changed', c)
         self.emit('map-marks-changed')
@@ -710,7 +738,7 @@ class Core(gobject.GObject):
 
     # called on signal by downloading thread
     def on_download_progress(self, something, cache_name, i, max_i):
-        self.gui.set_download_progress(float(i) / float(max_i), "Downloading %s (%d of %d)..." % (cache_name, i, max_i))
+        self.emit('progress', float(i) / float(max_i), "Downloading %s (%d of %d)..." % (cache_name, i, max_i))
         return False
 
     ##############################################
@@ -726,13 +754,13 @@ class Core(gobject.GObject):
         else:
             raise Exception("Format currently not supported: %s" % format)
 
-        self.gui.set_download_progress(0.5, "Exporting %s..." % cache.name)
+        self.emit('progress', 0.5, "Exporting %s..." % cache.name)
         try:
             exporter.export(cache, folder)
         except Exception, e:
-            self.gui.show_error(e)
+            self.emit('error', e)
         finally:
-            self.gui.hide_progress()
+            self.emit('hide-progress')
         
                 
 
@@ -754,7 +782,7 @@ class Core(gobject.GObject):
         self.emit('fieldnotes-changed')        
 
     def on_upload_fieldnotes(self):
-        self.gui.set_download_progress(0.5, "Uploading Fieldnotes...")
+        self.emit('progress', 0.5, "Uploading Fieldnotes...")
 
         caches = self.pointprovider.get_new_fieldnotes()
         fn = fieldnotesuploader.FieldnotesUploader(self.downloader)
@@ -769,14 +797,14 @@ class Core(gobject.GObject):
         for c in caches:
             fn.add_fieldnote(c)
         t = Thread(target=fn.upload)
-        t.daemon = False
+        t.daemon = True
         t.start()
         
     def on_upload_fieldnotes_finished(self, widget, caches):
         for c in caches:
             c.logas = geocaching.GeocacheCoordinate.LOG_NO_LOG
             self.save_cache_attribute(c, 'logas')
-        self.gui.hide_progress()
+        self.emit('hide-progress')
         self.emit('fieldnotes-changed')
 
     def get_new_fieldnotes_count(self):
@@ -804,9 +832,6 @@ class Core(gobject.GObject):
         self.emit('map-marks-changed')
 
 
-                
-    
-
 
 def determine_path ():
     """Borrowed from wxglade.py"""
@@ -823,6 +848,7 @@ def determine_path ():
                         
 
 def start():
+    gobject.threads_init()
     Core(gui, determine_path())
 
 def start_profile(what):
@@ -866,11 +892,12 @@ def start_profile(what):
             print "-- %d %4f %s" % (line.callcount, line.totaltime, line.code)
 
 
-if '--profile' in argv:
-    start = start_profile('start()')
 
         
 
 if __name__ == "__main__":
-    start()
+    if '--profile' in argv:
+        start_profile('start()')
+    else:
+        start()
 
