@@ -1,8 +1,8 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-#        Copyright (C) 2009 Daniel Fett
-#         This program is free software: you can redistribute it and/or modify
+#   Copyright (C) 2010 Daniel Fett
+#   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation, either version 3 of the License, or
 #   (at your option) any later version.
@@ -15,30 +15,13 @@
 #   You should have received a copy of the GNU General Public License
 #   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-#        Author: Daniel Fett advancedcaching@fragcom.de
+#   Author: Daniel Fett agtl@danielfett.de
+#   Jabber: fett.daniel@jaber.ccc.de
+#   Bugtracker and GIT Repository: http://github.com/webhamster/advancedcaching
 #
 
  
 # deps: python-html python-image python-netclient python-misc python-pygtk python-mime python-json
-# auto update feature
-# download map data
-# direction indicator in map view
-# edit waypoints
-# improved waypoint handling
-# - list of user waypoints:
-#   - per cache
-#   - "free" waypoints
-# - add/remove/edit of waypoints
-# - still parse waypoints from notes
-# - add parsing of calculations from notes
-# 'show on map' for waypoints
-# changes in menu:
-# - new "tools" button:
-#   - download map
-#   - upload fieldnotes
-#   - find location by name
-#   - export waypoints
-# add 'download map around geocaches' option
  
 ### For the gui :-)
 
@@ -64,10 +47,9 @@ import pango
 from portrait import FremantleRotation
 from simplegui import SimpleGui
 from simplegui import UpdownRows
-from simplegui import GeocacheLayer
-from simplegui import MarksLayer
 from xml.sax.saxutils import escape as my_gtk_label_escape
 from gtkmap import Map, OsdLayer, SingleMarkLayer
+from coordfinder import CalcCoordinate
 
 
 import logging
@@ -128,6 +110,9 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         self.core.connect('no-fix', self._on_no_fix)
         self.core.connect('settings-changed', self._on_settings_changed)
         self.core.connect('save-settings', self._on_save_settings)
+        self.core.connect('error', lambda caller, message: self.show_error(message))
+        self.core.connect('progress', lambda caller, fraction, text: self.set_progress(fraction, text))
+        self.core.connect('hide-progress', lambda caller: self.hide_progress())
 
         self.settings = {}
 
@@ -190,6 +175,8 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         self.icon_pixbufs = out
 
         self.image_icon_add = gtk.image_new_from_file(self.ICONPATH % {'size' : 64, 'name':'general_add'})
+        self.image_icon_add_2 = gtk.image_new_from_file(self.ICONPATH % {'size' : 64, 'name':'general_add'})
+        self.image_icon_add_3 = gtk.image_new_from_file(self.ICONPATH % {'size' : 64, 'name':'general_add'})
         self.image_zoom_in = gtk.image_new_from_file(self.ICONPATH % {'size' : 48, 'name':'pdf_zoomin'})
         self.image_zoom_out = gtk.image_new_from_file(self.ICONPATH % {'size' : 48, 'name':'pdf_zoomout'})
         self.image_action = gtk.image_new_from_file(self.ICONPATH % {'size' : 64, 'name':'keyboard_menu'})
@@ -202,9 +189,14 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
     def _open_browser(self, widget, link):
         system("dbus-send --print-reply --dest=com.nokia.osso_browser /com/nokia/osso_browser/request com.nokia.osso_browser.open_new_window 'string:%s' &" % link)
 
-    def show_coordinate_input(self, start, none_on_cancel=False):
+    def show_coordinate_input(self, start, none_on_cancel = False, show_name_input = False):
         udr = UpdownRows(self.format, start, True)
         dialog = gtk.Dialog("Edit Target", self.window, gtk.DIALOG_MODAL, ("OK", gtk.RESPONSE_ACCEPT))
+        if show_name_input:
+            e = hildon.Entry(gtk.HILDON_SIZE_AUTO)
+            dialog.vbox.pack_start(e)
+            e.set_text(start.name)
+
         dialog.set_size_request(-1, 480)
         dialog.vbox.pack_start(udr.table_lat)
                 
@@ -215,7 +207,10 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         dialog.destroy()
         if res == gtk.RESPONSE_ACCEPT:
             c = udr.get_value()
-            c.name = 'manual'
+            if show_name_input:
+                c.name = e.get_text()
+            else:
+                c.name = 'manual'
             return c
         elif none_on_cancel:
             return None
@@ -313,33 +308,10 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         self.main_gpspage.pack_start(self.main_gpspage_table, False, True)
         
         self.main_mappage = gtk.VBox()
-        try:
-            coord = geo.Coordinate(self.settings['map_position_lat'], self.settings['map_position_lon'])
-            zoom = self.settings['map_zoom']
-        except KeyError:
-            coord = self._get_best_coordinate(geo.Coordinate(50, 10))
-            zoom = 6
-
-        self.map = Map(center = coord, zoom = zoom)
-        self.geocache_layer = GeocacheLayer(self.core.pointprovider, self.show_cache)
-        self.marks_layer = MarksLayer()
-        self.map.add_layer(self.geocache_layer)
-        self.map.add_layer(self.marks_layer)
-        self.map.add_layer(OsdLayer())
-
-        self.core.connect('target-changed', self.marks_layer.on_target_changed)
-        self.core.connect('good-fix', self.marks_layer.on_good_fix)
-        self.core.connect('no-fix', self.marks_layer.on_no_fix)
-
-
-        self.map.connect('tile-loader-changed', lambda widget, loader: self._update_zoom_buttons())
-        #self.map.connect('map-dragged', lambda widget: self._set_track_mode(False))
-
+        self._configure_map()
 
         buttons = gtk.HBox()
 
-
-
         button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         button.set_image(self.image_action)
         button.connect("clicked", self._on_show_actions_clicked, None)
@@ -552,7 +524,7 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
                              'options_map_double_size': check_map_double_size.get_active(),
                              'options_hide_found': check_hide_found.get_active(),
                              })
-        self._on_save_settings(None)
+        self.core.save_settings(self.settings)
  
     def make_rearranging_table(self, elements, dialog, columns = 2):
         count = len(elements)
@@ -666,7 +638,7 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
                              #'options_rotate_screen': rotate_get_result(),
                              #'tts_interval':tts_get_result(),
                              })
-        self._on_save_settings(None)
+        self.core.save_settings(self.settings)
         #self.core.on_userdata_changed(self.settings['options_username'], self.settings['options_password'])
 
     def _get_tts_settings(self):
@@ -941,9 +913,17 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
             self.build_cache_images(cache, notebook)
 
             # calculated coords
-            cache.start_calc(text_longdesc)
-            if len(cache.calc.requires) > 0:
-                self.build_cache_calc(cache, notebook)
+            cache_calc_box = gtk.VBox()
+            notebook.append_page(cache_calc_box, gtk.Label("Calc"))
+
+            def rebuild_cache_calc():
+                cache.start_calc(self._strip_html(text_longdesc))
+                self.build_cache_calc(cache, cache_calc_box)
+
+            rebuild_cache_calc()
+            self.rebuild_cache_calc = rebuild_cache_calc
+
+
 
         # coords
         p = gtk.VBox()
@@ -960,8 +940,8 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         self.cache_notes.get_buffer().connect('changed', self.on_notes_changed)
         self.notes_changed = False
 
-        button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
-        button.set_label("Add Waypoint")
+        button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL	)
+        button.set_title("Add Waypoint")
         button.set_image(self.image_icon_add)
         button.connect("clicked", self._on_add_waypoint_clicked)
 
@@ -1058,7 +1038,11 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
             widget.realize()
 
     def build_coordinates(self, cache, p):
-
+        button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+        button.set_title("Add Coordinate")
+        button.set_image(self.image_icon_add_2)
+        button.connect("clicked", lambda caller, cache: self._add_waypoint_to_user_coordinates_list(cache), cache)
+        p.pack_start(button, False)
         def show_details(widget_coords, stuff, clist):
             c = clist[self._get_selected_pos(widget_coords)]
             if c == None:
@@ -1068,10 +1052,10 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
 
 
         p.pack_start(widget_coords, True, True)
-        widget_coords.show_all()
+        p.show_all()
 
     def __show_coordinate_details(self, c, cache):
-        RESPONSE_AS_TARGET, RESPONSE_AS_MAIN, RESPONSE_COPY_EDIT = range(3)
+        RESPONSE_AS_TARGET, RESPONSE_AS_MAIN, RESPONSE_COPY_EDIT, RESPONSE_EDIT, RESPONSE_DELETE = range(5)
         try:
             name = c.display_text
         except AttributeError:
@@ -1081,6 +1065,10 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         if c.lat != None:
             dialog.add_button("as Target", RESPONSE_AS_TARGET)
             dialog.add_button("as Main Coord.", RESPONSE_AS_MAIN)
+        if c.user_coordinate_id != None:
+            dialog.add_button("edit", RESPONSE_EDIT)
+            dialog.add_button("delete", RESPONSE_DELETE)
+        elif c.lat != None:
             dialog.add_button("copy & edit", RESPONSE_COPY_EDIT)
         lbl = gtk.Label()
         lbl.set_line_wrap(True)
@@ -1107,6 +1095,11 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         elif resp == RESPONSE_COPY_EDIT:
             self._add_waypoint_to_notes(c)
             self.update_coords()
+        elif resp == RESPONSE_EDIT:
+            self._on_edit_user_coordinate(None, cache, c.user_coordinate_id)
+            self.update_coords()
+        elif resp == RESPONSE_DELETE:
+            self.__delete_user_coordinate(cache, c.user_coordinate_id)
 
 
 
@@ -1144,13 +1137,14 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
 
         notebook.append_page(selector, gtk.Label("Images"))
 
-    def build_cache_calc(self, cache, notebook):
-
+    def build_cache_calc(self, cache, p):
+        logger.debug("Cache calc for %s" % p)
         def input_changed(widget, char):
             cache.calc.set_var(char, widget.get_text())
             self.show_cache_calc_results(cache)
 
-        p = gtk.VBox()
+        for x in p.get_children():
+            p.remove(x)
         count = len(cache.calc.requires)
         # create table with n columns.
         cols = 7
@@ -1176,56 +1170,174 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
             m.pack_start(e)
             table.attach(m, col, col + 1, row, row + 1)
             i += 1
+
+
+        # add button
+        button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL	)
+        button.set_title("Add Calculation")
+        button.set_image(self.image_icon_add_3)
+        button.connect("clicked", self._on_edit_user_coordinate, cache)
+        p.pack_start(button, False)
+
+        # add contents
         p.pack_start(table, False)
         p.pack_start(gtk.HSeparator(), False)
+
+        # add list vbox
         a = hildon.PannableArea()
-        vp = gtk.Viewport()
-        a.add(vp)
+        self.cache_calc_vbox = gtk.VBox()
+        a.add_with_viewport(self.cache_calc_vbox)
         p.pack_start(a, True)
-        self.cache_calc_viewport = vp
+
+
+        p.show_all()
         self.show_cache_calc_results(cache)
-        notebook.append_page(p, gtk.Label("Calc"))
 
     def show_cache_calc_results(self, cache):
-        p = gtk.VBox()
+        for x in self.cache_calc_vbox.get_children():
+            self.cache_calc_vbox.remove(x)
         vars = cache.calc.get_vars()
         for c in cache.calc.coords:
             if len(c.requires) == 0:
                 continue
+            if type(c.source) == int:
+                source = cache.get_user_coordinate(c.source)['name']
+            else:
+                source = c.source
             if c.has_requires():
                 text_calc = "= %s\n%s%s" % (c.replaced_result, c.result if c.result != False else '', "".join("\n<b>!</b> <span color='gold'>%s</span>" % warning for warning in c.warnings))
             else:
                 text_calc = "<i>Needs %s</i>\n" % (', '.join(("<s>%s</s>" if r in vars else "<b>%s</b>") % r for r in c.requires))
 
-            label_text = '<b>%s</b>\n%s' % (c.orig, text_calc)
-
+            label_text = '%s: <b>%s</b>\n%s' % (source, c.orig, text_calc)
+            #logger.debug("Showing cache calc: %s" % label_text)
             l = gtk.Label()
             l.set_alignment(0, 0.5)
             l.set_markup(label_text)
+            b = gtk.Table(rows = 2, columns = 2)
+            b.attach(l, 0, 1, 0, 2)
             
-            p.pack_start(l, False)
-            p.pack_start(gtk.HSeparator(), False)
+            button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+            button.set_label("edit")
+            button.connect("clicked", self._on_edit_user_coordinate, cache, c.source, c)
+            b.attach(button, 1, 2, 0, 1, 0, 0)
+
+            self.cache_calc_vbox.pack_start(b, False)
+            self.cache_calc_vbox.pack_start(gtk.HSeparator(), False)
             
-        for x in self.cache_calc_viewport.get_children():
-            self.cache_calc_viewport.remove(x)
-        self.cache_calc_viewport.add(p)
-        self.cache_calc_viewport.show_all()
+        self.cache_calc_vbox.show_all()
 
+    def _on_edit_user_coordinate(self, caller, cache, source_id = None, coordinate = None):
+        if source_id == None:
+            before_calc = ''
+            before_name = ''
+            ctype = geocaching.GeocacheCoordinate.USER_TYPE_CALC_STRING
+            logger.debug("Adding new calc string")
+            button_text = None
+        elif type(source_id) != int:
+            before_calc = coordinate.orig
+            before_name = ''
+            ctype = geocaching.GeocacheCoordinate.USER_TYPE_CALC_STRING_OVERRIDE
+            logger.debug("Overwriting original coordinate.")
+            button_text = None
+        else:
+            info = cache.get_user_coordinate(source_id)
+            before_name = info['name']
+            ctype = info['type']
+            if info['type'] == geocaching.GeocacheCoordinate.USER_TYPE_CALC_STRING:
+                before_calc = info['value']
+                logger.debug("Editing user supplied calc string.")
+                button_text = 'Delete'
+            elif info['type'] == geocaching.GeocacheCoordinate.USER_TYPE_CALC_STRING_OVERRIDE:
+                before_calc = info['value'][1]
+                logger.debug("Editing user supplied overwrite.")
+                button_text = 'Reset'
+            elif info['type'] == geocaching.GeocacheCoordinate.USER_TYPE_COORDINATE:
+                start = geo.Coordinate(info['value'][0], info['value'][1], info['name'])
+                res = self.show_coordinate_input(start, none_on_cancel = True, show_name_input = True)
+                if res == None:
+                    return
+                cache.set_user_coordinate(geocaching.GeocacheCoordinate.USER_TYPE_COORDINATE, (res.lat, res.lon), res.name, source_id)
+                self.core.save_cache_attribute(self.current_cache, 'user_coordinates')
+                self.update_coords()
+                return
+            else:
+                raise Exception("Illegal type.")
 
+        (res, after_name, after_calc) = self.__show_edit_calculation(before_calc, before_name, button_text)
+
+        if res not in (gtk.RESPONSE_ACCEPT, gtk.RESPONSE_NO):
+            return
+        if res == gtk.RESPONSE_ACCEPT:
+            try:
+                geo.try_parse_coordinate(after_calc)
+                self.show_success("Coordinate saved.")
+            except Exception:
+                self.show_success("Calc string saved.")
+            if source_id == None:
+                value = after_calc
+                id = None
+            elif type(source_id) != int:
+                value = (coordinate.signature, after_calc)
+                id = None
+            elif info['type'] == geocaching.GeocacheCoordinate.USER_TYPE_CALC_STRING or info['type'] == geocaching.GeocacheCoordinate.USER_TYPE_COORDINATE:
+                value = after_calc
+                id = source_id
+            else:
+                value = (info['value'][0], after_calc)
+                id = source_id
+            cache.set_user_coordinate(ctype, value, after_name, id)
+        elif res == gtk.RESPONSE_NO:
+            cache.delete_user_coordinate(source_id)
+
+        self.core.save_cache_attribute(self.current_cache, 'user_coordinates')
+        self.rebuild_cache_calc()
+        
+    def __delete_user_coordinate(self, cache, id):
+        cache.delete_user_coordinate(id)
+        self.core.save_cache_attribute(cache, 'user_coordinates')
+        self.rebuild_cache_calc()
+        self.update_coords()
+
+    def __show_edit_calculation(self, before_calc, before_name, button_text):
+        dialog = gtk.Dialog("Edit Calculation", self.window, gtk.DIALOG_DESTROY_WITH_PARENT, (
+            gtk.STOCK_OK, gtk.RESPONSE_ACCEPT
+            ))
+
+        if button_text != None:
+            dialog.add_button(button_text, gtk.RESPONSE_NO)
+
+        text = "Enter a Coordinate or a calculation string.\n" + \
+        "Format is: <b>N12 12.123 E123 12.123</b>\n" + \
+        "Variables: <b>A-Z</b>, Operators: <b>+-*/%()</b>"
+        lbl = gtk.Label()
+        dialog.vbox.pack_start(lbl)
+        lbl.set_line_wrap(True)
+        lbl.set_alignment(0, 0.5)
+        lbl.set_markup(text)
+
+        grp = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+        entry_name = hildon.Entry(gtk.HILDON_SIZE_AUTO)
+        entry_name.set_text(before_name)
+        dialog.vbox.pack_start(hildon.Caption(grp, "Name", entry_name, None, hildon.CAPTION_MANDATORY))
+
+        entry_calc = hildon.Entry(gtk.HILDON_SIZE_AUTO)
+        entry_calc.set_text(before_calc)
+        dialog.vbox.pack_start(hildon.Caption(grp, "Calculation", entry_calc, None, hildon.CAPTION_MANDATORY))
+
+        dialog.show_all()
+        while True:
+            res = dialog.run()
+            if res == gtk.RESPONSE_ACCEPT and not CalcCoordinate.is_calc_string(entry_calc.get_text()):
+                self.show_error("Please enter a valid calculation.")
+            else:
+                break
+        dialog.hide()
+        return (res, entry_name.get_text(), entry_calc.get_text())
+        
     def _on_add_waypoint_clicked (self, widget):
         self._add_waypoint_to_notes()
 
-    def _get_best_coordinate(self, start=None):
-        if start != None:
-            c = start
-        elif self.gps_data != None and self.gps_data.position != None:
-            c = self.gps_data.position
-        elif self.core.current_target != None:
-            c = self.core.current_target
-        else:
-            c = geo.Coordinate(0, 0)
-        return c
-
     def _add_waypoint_to_notes(self, start=None):
         res = self.show_coordinate_input(self._get_best_coordinate(start), none_on_cancel = True)
         if res == None:
@@ -1233,6 +1345,16 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         text = "\n%s\n" % res.get_latlon(self.format)
         self.cache_notes.get_buffer().insert(self.cache_notes.get_buffer().get_end_iter(), text)
 
+
+
+    def _add_waypoint_to_user_coordinates_list(self, cache, start=None):
+        res = self.show_coordinate_input(self._get_best_coordinate(start), none_on_cancel = True, show_name_input = True)
+        if res == None:
+            return
+        cache.set_user_coordinate(geocaching.GeocacheCoordinate.USER_TYPE_COORDINATE, (res.lat, res.lon), res.name)
+        self.core.save_cache_attribute(self.current_cache, 'user_coordinates')
+        self.update_coords()
+
         
     def _on_show_image(self, dpath, caption):
         fullpath = path.join(self.settings['download_output_dir'], dpath)
@@ -1365,9 +1487,6 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         self.hide_cache_view(go_to_map = True)
         self.set_active_page(True)
 
-    def set_target(self, cache):
-        self.core.set_target(cache)
-
     def _on_target_changed(self, caller, cache, distance, bearing):
         self.gps_target_distance = distance
         self.gps_target_bearing = bearing
@@ -1425,9 +1544,7 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         b = self.cache_notes.get_buffer()
         return b.get_text(b.get_start_iter(), b.get_end_iter())
                 
-
-    #called by core
-    def set_download_progress(self, fraction, text = ''):
+    def set_progress(self, fraction, text = ''):
         hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
         if text == '':
             text = 'Please wait...'
@@ -1437,12 +1554,11 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
             self.banner.show_all()
         else:
             self.banner.set_text(text)
+        gtk.gdk.threads_leave()
         
 
 
     def show_error(self, errormsg):
-        #if isinstance(errormsg, Exception):
-        #    raise errormsg
         hildon.hildon_banner_show_information(self.window, "", "%s" % errormsg)
 
     def show_success(self, message):
@@ -1476,7 +1592,7 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
 
 
         if self.gps_has_fix and self.gps_target_distance != None:
-            td_string = self._format_distance(self.gps_target_distance)
+            td_string = geo.Coordinate.format_distance(self.gps_target_distance)
             self.label_dist.set_markup("<span size='xx-large'>%s</span>" % td_string)
         elif self.gps_target_distance == None:
             self.label_dist.set_markup("<span size='x-large'>No Target</span>")
@@ -1500,37 +1616,11 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
         #
         ##############################################
 
-        
-
-    def _on_settings_changed(self, caller, settings, source):
-        #if source == self:
-        #    return
-        self.settings.update(settings)
-
-        self.block_changes = True
-        if 'options_hide_found' in settings:
-            self.geocache_layer.set_show_found(not settings['options_hide_found'])
-        if 'options_show_name' in settings:
-            self.geocache_layer.set_show_name(settings['options_show_name'])
-        if 'options_map_double_size' in settings:
-            self.map.set_double_size(settings['options_map_double_size'])
-        if 'map_zoom' in settings:
-            if self.map.get_zoom() != settings['map_zoom']:
-                self.map.set_zoom(settings['map_zoom'])
-        if 'map_position_lat' in settings and 'map_position_lon' in settings:
-            self.set_center(geo.Coordinate(settings['map_position_lat'], settings['map_position_lon']), reset_track = False)
-        if 'map_follow_position' in settings:
-            self._set_track_mode(settings['map_follow_position'])
+    def _on_settings_changed_gui(self, settings):
         if 'options_rotate_screen' in settings:
             self.rotation_manager.set_mode(settings['options_rotate_screen'])
-        if 'last_target_lat' in settings:
-            self.set_target(geo.Coordinate(settings['last_target_lat'], settings['last_target_lon']))
-        if 'last_selected_geocache' in settings and settings['last_selected_geocache'] not in (None, ''):
-            cache = self.core.get_geocache_by_name(settings['last_selected_geocache'])
-            if cache != None:
-                self.set_current_cache(cache)
+        
 
-        self.block_changes = False
 
     def _on_save_settings(self, caller):
         c = self.map.get_center()
@@ -1545,4 +1635,4 @@ class HildonGui(HildonSearchPlace, HildonFieldnotes, HildonSearchGeocaches, Hild
 
         for i in ['options_username', 'options_password', 'download_noimages', 'options_show_name', 'options_hide_found', 'options_show_html_description', 'options_map_double_size', 'options_rotate_screen', 'tts_interval']:
             settings[i] = self.settings[i]
-        self.core.save_settings(settings, self)
+        caller.save_settings(settings, self)
